"""The client module defines stream and query based clients.

Stream clients allow to read and write from stream based sources with ease,
and in an interchangable manner.

Query based clients unify how to access databases leveraging from sqlalchemy.
"""

from .databricks_client import *  # noqa



"""tentaclio is a library that tries to unify how to load, store data.
The main use cases in mind are ETL processes and notebooks, but can be used in many other contexts.
The main benefits are:
    * url based resource management.
    * same function to open readers and writers for resources of different natures.
    Just change the url, the code remains the same.
    * The same for dbs, create clients with ease and use them regardless
    the underlying implementation (thanks to sqlalchemy).
    * Credentials management that allows a distributed credentials storage.
"""
from tentaclio import *  # noqa

from .clients.databricks_client import DatabricksClient


# Add DB registry
DB_REGISTRY.register("databricks+pyodbc", DatabricksClient)  # type: ignore



#!/usr/bin/env python
# -*- coding: utf-8 -*

import os
import pathlib

from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

VERSION = "0.0.0.1"

REPO_ROOT = pathlib.Path(__file__).parent

# Fetch the long description from the readme
with open(REPO_ROOT / "README.md", encoding="utf-8") as f:
    README = f.read()

install_requires = [
    "tentaclio",
    "pyodbc",
    "psycopg2",
]


setup_args = dict(
    name="tentaclio-databricks",
    version=VERSION,
    include_package_data=True,
    description="A python project containing all the dependencies for schema databricks:pyodbc for tentaclio.",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Octopus Energy",
    author_email="nerds@octopus.energy",
    license="Proprietary",
    package_dir={"": "src"},
    packages=find_packages("src", include=["*tentaclio_databricks*"]),
    install_requires=install_requires,
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)


if __name__ == "__main__":
    setup(**setup_args)



"""Databricks query client."""
import urllib
from typing import Dict

from sqlalchemy.engine import Connection, Engine, create_engine
from tentaclio.clients import sqla_client


class DatabricksClient(sqla_client.SQLAlchemyClient):
    """Databricks client, backed by a pyodbc + SQLAlchemy connection."""

    def _connect(self) -> Connection:
        odbc_connection_map = self._build_odbc_connection_dict()
        connection_url = build_odbc_connection_string(**odbc_connection_map)

        if self.engine is None:
            self.engine: Engine = create_engine(f"mssql+pyodbc:///?odbc_connect={connection_url}")
        return self.engine.connect()

    def _build_odbc_connection_dict(self) -> Dict:
        odbc_connection_string_map = {
            "UID": "token",
            "PWD": self.username,
            "HOST": self.host,
            "PORT": self.port,
            "Schema": self.database,
        }
        if self.url.query:
            odbc_connection_string_map.update(self.url.query)
        return odbc_connection_string_map


def build_odbc_connection_string(**kwargs) -> str:
    """Build a url formatted odbc connection string from kwargs."""
    connection_url = ";".join([f"{k}={v}" for k, v in kwargs.items()])
    return urllib.parse.quote(connection_url)
